// *****************************************************************************
//  Copyright (C): Christian Nastasi, 2011                                    
//  Author(s): Christian Nastasi                                              
//  Developed at the:                                                         
//  - Retis LAB, TeCIP Institute, Scuola Superiore Sant'Anna (Pisa)           
//  - School of Elec Eng and Computer Science, Queen Mary University (London) 
//  This file is distributed under the terms in the attached LICENSE_2 file.  
//  If you do not find this file, copies can be found by writing to:          
//  - c.nastasi@sssup.it                                                      
//  - nastasichr@gmail.com                                                    
//  - andrea.cavallaro@eecs.qmul.ac.uk                                        
// *****************************************************************************

#ifndef __WiseBaseApplication_h__
#define __WiseBaseApplication_h__

#include <sstream>
#include <vector>
#include <string>
#include <omnetpp.h>
#include "WiseApplicationPacket_m.h"
#include "WiseSensorManagerMessage_m.h"
#include "ResourceManager.h"
#include "Radio.h"
#include "VirtualMobilityManager.h"
#include "CastaliaModule.h"
#include "TimerService.h"

#define SELF_NETWORK_ADDRESS selfAddress.c_str()

using namespace std;

class WiseBaseApplication: public CastaliaModule, public TimerService {
private:
	simtime_t initialize_time;
	bool first_initialize;

protected:
	/*--- The .ned file's parameters ---*/
	string applicationID;
	int priority;
	bool synchronizedStartup;
	int maxAppPacketSize;
	int packetHeaderOverhead;
	int constantDataPayload;
	bool isSink;
	double latencyMax;
	int latencyBuckets;

	/*--- Custom class parameters ---*/
	int self;			// the node's ID
	int n_nodes;			// number of nodes
	string selfAddress;
	ResourceManager *resMgrModule;
	VirtualMobilityManager *mobilityModule;	
	Radio *radioModule;
	int disabled;
	double cpuClockDrift;
	bool neighbour_discover;
	std::vector<std::string> neighbours;
	double last_packet_latency;

protected:
	WiseBaseApplication() : first_initialize(true) {} ;
	virtual void initialize();
	virtual void startup() {} 
	virtual void handleMessage(cMessage * msg);
	virtual void finish();
	virtual void finishSpecific() {}

	void requestSensorReading(int index = 0);
	void toNetworkLayer(cMessage *);
	void toNetworkLayer(cPacket *, const char *, double delay=0);
	void sendDirectApplicationMessage(WiseApplicationPacket*, const char*,
				   unsigned type=WISE_APP_DIRECT_COMMUNICATION);

	virtual void fromNetworkLayer(WiseApplicationPacket *, const char *,
				      double, double) = 0;
	virtual void handleSensorReading(WiseSensorManagerMessage *) {};
	virtual void handleNetworkControlMessage(cMessage *) {};
	virtual void handleMacControlMessage(cMessage *) {};
	virtual void handleRadioControlMessage(RadioControlMessage *) {};
	virtual void handleDirectApplicationMessage(WiseApplicationPacket *) {};
	virtual void notifyNeighbourDiscovered() {};

	inline bool lastNode() const { return self == (n_nodes - 1); } ;
	inline bool firstNode() const { return self == 0; } ;

private:
	void readParameter();
	void processAppPacket(WiseApplicationPacket *pkt);
	void normalAppPacket(WiseApplicationPacket *pkt);
	void neighbourDiscover(double);
	void procNeighbourDiscoverRequest(WiseApplicationPacket *pkt);
	void procNeighbourDiscoverReply(WiseApplicationPacket *pkt);
	void neighbourDiscoverCompleted();
};

#endif //__WiseBaseApplication_h__
